function OUTPUT = ann_calc_output(INP2HID, HID2OUT, INPUT)
%ANN_CALC_OUTPUT - calculates the output of a three stage artificial neural network
% INP2HID: input to hidden layer weights
% HID2OUT: hidden layer to output weights
% INPUT: input vector
% OUTPUT: output vector

HID_LAYER_IN    = INP2HID * [INPUT; ones(1, size(INPUT,2))]; % the 1's are there because we also allow a bias node
HID_LAYER_OUT   = sigmoid(HID_LAYER_IN);

OUTPUT_LAYER_IN = HID2OUT * [HID_LAYER_OUT; ones(1, size(HID_LAYER_OUT,2))];
OUTPUT          = sigmoid(OUTPUT_LAYER_IN);


%% SUBFUNCTION sigmoid
function Y = sigmoid(X)
% sigmoidal activation function
% see Haykin: Adaptive Filter Theory, pp. 736 - 760
	Y = 1./(1+exp(-X));
    