function [TRAINING_LENGTH, CLASSIFICATION_LENGTH, NUM_CLASSES, CLASSES, FEATURE_LENGTH] = get_lengths(DATA, PARAMETERS)
%GET_LENGTHS - calculates the number of training and classification sets
%              and all class labels
%
%  TRAINING_LENGTH       - number of training patterns
%  CLASSIFICATION_LENGTH - number of classification patterns
%  NUM_CLASSES           - number of classes
%  CLASSES               - class labels
%  FEATURE_LENGTH        - number of features
%
% these calculations are needed for all classifiers


if exist('DATA','var') == 1 && isfield(DATA,'TRAINING') == 1
    TRAINING_LENGTH = size(DATA.TRAINING,1);
    FEATURE_LENGTH  = size(DATA.TRAINING,2)-1;
else
    TRAINING_LENGTH = 0;
end

if exist('DATA','var') == 1 && isfield(DATA,'FEATURES') == 1
        FEATURE_LENGTH = size(DATA.FEATURES,2) - 1;
end

if exist('DATA','var') == 1 && isfield(DATA,'CLASSIFICATION') == 1
    CLASSIFICATION_LENGTH = size(DATA.CLASSIFICATION,1);
    FEATURE_LENGTH  = size(DATA.CLASSIFICATION,2)-1;
else
    CLASSIFICATION_LENGTH = 0;
end


%% Determine CLASSES and NUM_CLASSES
% CLASSES is a unique matrix with all possible class labels
CLASSES = [];
if exist('DATA','var') == 1 && isfield(DATA,'TRAINING') == 1
    CLASSES = unique(DATA.TRAINING(:,end));
else
    if exist('DATA','var') == 1 && isfield(DATA,'CLASSIFICATION') == 1
        CLASSES = unique([CLASSES ; DATA.CLASSIFICATION(:,end)]);
    end
end

if exist('DATA','var') == 1 && isfield(DATA,'FEATURES') == 1
    CLASSES = unique(DATA.FEATURES(:,end));
end


NUM_CLASSES = length(CLASSES); % Number of classes