function EVAL_MATRIX = k_nearest_neighbours_classifier(DATA, PARAMETERS)
%K_NEAREST_NEIGHBOURS_CLASSIFIER - this function uses the k nearest neighbours for
% classification of all data in FEATURE_MATRIX_CLASSIFICATION
% Possible parameters are:
%
% PARAMETERS.NEAREST_NEIGHBOURS - number of nearest neighbours that should considered
% PARAMETERS.METRIC             - sets the used metric. May be 'euclidian' or 'mahalanobis'
%
% Note: The k-nearest neighbour classifier is sensitive to the scaling of the features.
%       It is therefore advantageous to normalize the input features.
%% 1. INITIALISATION
[TRAINING_LENGTH, CLASSIFICATION_LENGTH, NUM_CLASSES, CLASSES, FEATURE_LENGTH] = get_lengths(DATA, PARAMETERS);

% initialize evaluation matrix
EVAL_MATRIX = [zeros(CLASSIFICATION_LENGTH, 1) DATA.CLASSIFICATION(:, end)];

% if PARAMETERS.NEAREST_NEIGHBOURS is not set than use only the nearest one,
% i.e. PARAMETERS.NEAREST_NEIGHBOURS = 1
if exist('PARAMETERS','var') == 0 || isfield(PARAMETERS,'NEAREST_NEIGHBOURS') == 0
    PARAMETERS.NEAREST_NEIGHBOURS = 1;
end

if exist('PARAMETERS','var') == 0 || isfield(PARAMETERS,'METRIC') == 0
    PARAMETERS.METRIC = 'euclidian';
end


%% 2. TRAINING PHASE
% for this algorithm, we don't need a trainings phase




%% 3. CLASSIFICATION PHASE

if strcmp(PARAMETERS.METRIC,'mahalanobis')
    S = cov(DATA.TRAINING(:, 1:FEATURE_LENGTH)); %scatter matrix
    if min(eig(S)) > 0
        L = chol(S);
    else
        fprintf('using euclidian distance because S not positive definit.\n')
        PARAMETERS.METRIC = 'euclidian';
    end
end

for ROW = 1:CLASSIFICATION_LENGTH
    % calculate x^T*x for each row
    % therefore expand the actual feature vector to a matrix
    A = DATA.TRAINING(:, 1:FEATURE_LENGTH) - repmat(DATA.CLASSIFICATION(ROW, 1:FEATURE_LENGTH), TRAINING_LENGTH, 1);
    if strcmp(PARAMETERS.METRIC,'mahalanobis')
        B = A / L;
        DISTANCES = sum(B.*B,2);
    else
        DISTANCES = sum(A.*A,2);
    end

    [DISTANCES_SORTED, INDICES] = sort(DISTANCES);
    [VAL, INDEX] = max(hist(DATA.TRAINING(INDICES(1:PARAMETERS.NEAREST_NEIGHBOURS),end), CLASSES(1:NUM_CLASSES)));
    EVAL_MATRIX(ROW, 1) = CLASSES(INDEX);
end
