function [EVAL_MATRIX, PROB_EVAL_MATRIX] = linear_discriminant_function_classifier(DATA, PARAMETERS)
%LINEAR_DISCRIMINANT_FUNCTION_CLASSIFIER
% this function realizes a linear discriminant classifier
% which uses a minimum squared error criterion to obtain the
% weigthes of the vector a
%
% It returns a matrix EVAL_MATRIX which contains in the first column
% the estimated classes and in the second one the original classes.
% Additionally it returns a PROB_EVAL_MATRIX where each row contains the
% probabilities for one data sample of the classification set.
% See Duda how the discriminant function is linked to the a posteriori probability
% 
% Possible parameters are:
%
% NONE

%% 1. INITIALISATION
[TRAINING_LENGTH, CLASSIFICATION_LENGTH, NUM_CLASSES, CLASSES, FEATURE_LENGTH] = get_lengths(DATA, PARAMETERS);

TR.TRAINING = DATA.TRAINING;

% Transform emotions into indices of CLASSES
for CLASSES_C = 1:NUM_CLASSES
    INDICES{CLASSES_C} = DATA.TRAINING(:,end) == CLASSES(CLASSES_C);
end
for CLASSES_C = 1:NUM_CLASSES
    TR.TRAINING(INDICES{CLASSES_C},end) = CLASSES_C;
end

%% 2. TRAININGS PHASE
H_MATRIX = [ones(TRAINING_LENGTH, 1) DATA.TRAINING(:, 1:FEATURE_LENGTH)];
B_VEC    = zeros(TRAINING_LENGTH, NUM_CLASSES);  % predefine vector b with zeros

% turn 0 in 1 in each row for the correct class
B_VEC(sub2ind(size(B_VEC), 1:TRAINING_LENGTH, TR.TRAINING(:, end)')) = 1;

% calculate pseudo-inverse
% do not use inv(A'*A)*A' as pinv() is numerically stabler
WEIGHTS = pinv(H_MATRIX)*B_VEC;



%% 3. CLASSIFICATION PHASE
DISCRIMINANT_FUNCTIONS = [ones(CLASSIFICATION_LENGTH, 1) DATA.CLASSIFICATION(:,1:FEATURE_LENGTH)] * WEIGHTS;
[VAL, INDEX] = max(DISCRIMINANT_FUNCTIONS, [], 2);
EVAL_MATRIX = [CLASSES(INDEX) DATA.CLASSIFICATION(:, end)];

% value of discriminant functions can be interpreted as a posteriori probabilities
PROB_EVAL_MATRIX = DISCRIMINANT_FUNCTIONS;