function [EVAL_MATRIX, PROB_EVAL_MATRIX] = linear_discriminant_function_classifier_quadratic(DATA, PARAMETERS)
% LINEAR_DISCRIMINANT_FUNCTION_CLASSIFIER_QUADRATIC - realizes a linear discriminant classifier
% which uses a minimum squared error criterion to obtain the
% weigthes of the vector a
%
% This second version uses a quadratic discriminant function
% to obtain better results for the classification error
%
% It returns a matrix EVAL_MATRIX which contains in the first column
% the estimated classes and in the second one the original classes.
% Additionally it returns a PROB_EVAL_MATRIX where each row contains the
% probabilities for one data sample of the classification set.
% See Duda how the discriminant function is linked to the a posteriori probability
% 
% Possible parameters are:
%
% NONE

%% 1. INITIALISATION
[TRAINING_LENGTH, CLASSIFICATION_LENGTH, NUM_CLASSES, CLASSES, FEATURE_LENGTH] = get_lengths(DATA, PARAMETERS);

% needed for the transformation of the feature vector
if FEATURE_LENGTH > 1
    PERMUTATIONS = [(1:FEATURE_LENGTH)' (1:FEATURE_LENGTH)' ; nchoosek(1:FEATURE_LENGTH, 2)];
else
    PERMUTATIONS = [1 1];
end

TR.TRAINING = DATA.TRAINING;

% Transform emotions into indices of CLASSES
for CLASSES_C = 1:NUM_CLASSES
    INDICES{CLASSES_C} = DATA.TRAINING(:,end) == CLASSES(CLASSES_C);
end
for CLASSES_C = 1:NUM_CLASSES
    TR.TRAINING(INDICES{CLASSES_C},end) = CLASSES_C;
end

%% 2. TRAININGS PHASE
H_MATRIX = transform_feature_vector(PERMUTATIONS, DATA.TRAINING(:, 1:FEATURE_LENGTH));
B_VEC    = zeros(TRAINING_LENGTH, NUM_CLASSES);  % predefine vector b with 0

% turn 0 in 1 in each row for the correct class
B_VEC(sub2ind(size(B_VEC), 1:TRAINING_LENGTH, TR.TRAINING(:, end)')) = 1;

% calculate pseudo-inverse of H_MATRIX and use this to obtain the WEIGHTS
WEIGHTS = pinv(H_MATRIX)*B_VEC;



%% 3. CLASSIFICATION PHASE
DISCRIMINANT_FUNCTIONS = transform_feature_vector(PERMUTATIONS,DATA.CLASSIFICATION(:,1:FEATURE_LENGTH)) * WEIGHTS;
[VAL, INDEX] = max(DISCRIMINANT_FUNCTIONS,[],2);
EVAL_MATRIX  = [CLASSES(INDEX) DATA.CLASSIFICATION(:, end)];

% a posteriori probabilities
PROB_EVAL_MATRIX = DISCRIMINANT_FUNCTIONS;





%% SUBFUNCTION transform_feature_vector
function Y = transform_feature_vector(PERMUTATIONS, X)
% subfunction transform feature vector
% transforms the feature vector into a quadratic form
%
Y = [ones(size(X,1),1) X X(:,PERMUTATIONS(:,1)).*X(:,PERMUTATIONS(:,2))];