function EVAL_MATRIX = nearest_mean_classifier(DATA, PARAMETERS)
%NEAREST_MEAN_CLASSIFIER - this function uses the nearest mean for the classification
% It returns a matrix EVAL_MATRIX which contains in the first column
% the estimated classes and in the second one the original classes.
% Possible parameters are:
%
% NONE

%% 1. INITIALISATION
[TRAINING_LENGTH, CLASSIFICATION_LENGTH, NUM_CLASSES, CLASSES, FEATURE_LENGTH] = get_lengths(DATA, PARAMETERS);

% initialize evaluation matrix
EVAL_MATRIX = [zeros(CLASSIFICATION_LENGTH, 1) DATA.CLASSIFICATION(:, end)];



%% 2. TRAINING PHASE
% calculate the means for all classes. They represent the whole class
MEAN_VALUES = zeros(NUM_CLASSES, FEATURE_LENGTH);

for CLASSES_C = 1:NUM_CLASSES
    INDICES = DATA.TRAINING(:, end) == CLASSES(CLASSES_C);
    MEAN_VALUES(CLASSES_C, :) = mean(DATA.TRAINING(INDICES, 1:FEATURE_LENGTH));
end



%% 3. CLASSIFICATION
for ROW = 1:CLASSIFICATION_LENGTH % perhaps it is faster to loop over NUM_CLASSES and not over CLASSIFICATION_LENGTH...
    % calculate x^T*x for each row
    % therefore expand the actual feature vector to a matrix
    A = MEAN_VALUES - repmat(DATA.CLASSIFICATION(ROW, 1:FEATURE_LENGTH), NUM_CLASSES, 1);
    B = A.*A;
    DISTANCES = sum(B,2);
    [DISTANCES_MIN, INDEX] = min(DISTANCES);
    EVAL_MATRIX(ROW, 1) = CLASSES(INDEX);
end
