function [DATA2, DATA3, W1, W2] = fisher_discriminant_analysis(DATA, DIM)
%FISHER_DISCRIMINANT_ANALYSIS
% This function determines the direction where the fisher
% discriminant is maximized (determinant criterion)
%
% DATA2 uses S_B in the numerator
% DATA3 uses S_T in the numerator
%
% it is stated in the literature that DATA3 is somewhat more stable for
% small sample sizes than DATA2, but the effect is rather small and could not be observed ...
%
% Make sure that there are no features which have a small standard deviation as this causes the fisher
% projection to choose this direction independent of whether it is good or not

%% 1. INITIALISATION
[TRAINING_LENGTH, CLASSIFICATION_LENGTH, NUM_CLASSES, CLASSES, FEATURE_LENGTH] = get_lengths(DATA, []);


if  FEATURE_LENGTH < DIM
    %       fprintf('Dimension is already smaller to new dimension. No calculation necessary\n');
    DATA2 = DATA;
    DATA3 = DATA;
    return
end

% determine class-related values
for CLASSES_C = 1:NUM_CLASSES
    INDICES = find(DATA.TRAINING(:, end) == CLASSES(CLASSES_C));
    THETA(CLASSES_C).N = length(INDICES);                                   % number of samples for each class
    THETA(CLASSES_C).M = mean(DATA.TRAINING(INDICES, 1:FEATURE_LENGTH));	% mean of all samples for each class
    THETA(CLASSES_C).S = DATA.TRAINING(INDICES, 1:FEATURE_LENGTH)'*DATA.TRAINING(INDICES, 1:FEATURE_LENGTH); %scatter matrix
end

% calculate total mean
M = mean(DATA.TRAINING(:, 1:FEATURE_LENGTH));

S_W = zeros(FEATURE_LENGTH);
S_B = zeros(FEATURE_LENGTH);

for CLASSES_C = 1:NUM_CLASSES
    S_W = S_W + THETA(CLASSES_C).S;
    S_B = S_B + THETA(CLASSES_C).N*(THETA(CLASSES_C).M - M)'*(THETA(CLASSES_C).M - M);
end
S_T = S_W + S_B;

% 1. Possibility: use S_B
[V, D] = eig(S_B, S_W);
[VAL, INDEX] = sort(diag(D));
W1 = V(:, INDEX(end-DIM+1:end)); % (d x NEW_DIMENSION dimensional)

% 2. Possibility: use S_T
[V, D] = eig(S_T, S_W);
[VAL, INDEX] = sort(diag(D));
W2 = V(:, INDEX(end-DIM+1:end)); % (d x NEW_DIMENSION dimensional)

DATA2.TRAINING       = [DATA.TRAINING(:,1:FEATURE_LENGTH)*W1       DATA.TRAINING(:, end)];
if isfield(DATA,'CLASSIFICATION')
    DATA2.CLASSIFICATION = [DATA.CLASSIFICATION(:,1:FEATURE_LENGTH)*W1 DATA.CLASSIFICATION(:, end)];
end

DATA3.TRAINING       = [DATA.TRAINING(:,1:FEATURE_LENGTH)*W2       DATA.TRAINING(:, end)];
if isfield(DATA,'CLASSIFICATION')
    DATA3.CLASSIFICATION = [DATA.CLASSIFICATION(:,1:FEATURE_LENGTH)*W2 DATA.CLASSIFICATION(:, end)];
end

