function [DATA_NEW, W] = principal_component_analysis(DATA, DIM)
%PRINCIPLE_COMPONENT_ANALYSIS - calculates the principle components and selects the first DIM eigenvectors
% which belong to the largest eigenvalues, i.e. we choose the directions which
% show the greatest scatter cloud.
%
% It returns the transformed data

[TRAINING_LENGTH, CLASSIFICATION_LENGTH, NUM_CLASSES, CLASSES, FEATURE_LENGTH] = get_lengths(DATA, []);

if  FEATURE_LENGTH < DIM
    %       fprintf('Dimension of Feature vector is already smaller to new dimension. No calculation necessary\n');
    DATA_NEW = DATA;
    return
end

BIAS = mean(DATA.TRAINING(:, 1:FEATURE_LENGTH),1);

% Instead of calculating the eigenvectors in the covariance domain, we use
% the SVD to calculate the eigenvectors in the data domain which is numerically more robust
[U, S, V] = svd(DATA.TRAINING(:,1:FEATURE_LENGTH) - repmat(BIAS, TRAINING_LENGTH, 1),'econ');

DATA_NEW.TRAINING       = bsxfun(@times, U(:,1:DIM), diag(S(1:DIM,1:DIM)));
DATA_NEW.TRAINING       = [DATA_NEW.TRAINING DATA.TRAINING(:, end)];

% transformation matrix
W = V(:,1:DIM);

if isfield(DATA,'CLASSIFICATION')
    DATA_NEW.CLASSIFICATION = (DATA.CLASSIFICATION(:,1:FEATURE_LENGTH) - repmat(BIAS, CLASSIFICATION_LENGTH, 1)) * W;
    DATA_NEW.CLASSIFICATION = [DATA_NEW.CLASSIFICATION DATA.CLASSIFICATION(:, end)];
end



