function [CONF_MATRIX, LATEX_TABULAR] = generate_confusion_matrix(EVAL_MATRIX)
%GENERATE_CONFUSION_MATRIX
% generates out of a evaluation matrix the corresponding confusion
% matrix
% the row contains the actual class and the column the determined/estimated class
%
% LATEX_TABULAR can be used to automatically generate a tabular out of the
% confusion matrix. This might be helpful for writing a papers or a thesis.
% fprintf(LATEX_TABULAR) writes the Latex code into the shell

% we estimate the number of classes by counting all different class IDs
if ~all(all(isnan(EVAL_MATRIX) == 0))
    warning('EVAL_MATRIX contains NaNs!!')
    CONF_MATRIX = [];
    LATEX_TABULAR = [];
    return
end

CLASSES     = unique([EVAL_MATRIX(:,1);EVAL_MATRIX(:,2)]);
NUM_CLASSES = length(CLASSES);

CONF_MATRIX = zeros(NUM_CLASSES+1);
CONF_MATRIX(2:end,1) = CLASSES;
CONF_MATRIX(1,2:end) = CLASSES;

% transform class labels to indices which are used in CLASSES
for CLASSES_C = 1:NUM_CLASSES
    INDICES1{CLASSES_C} = EVAL_MATRIX(:,1) == CLASSES(CLASSES_C); % first get ALL indices before
    INDICES2{CLASSES_C} = EVAL_MATRIX(:,2) == CLASSES(CLASSES_C); % replacing with CLASSES_C
end

for CLASSES_C = 1:NUM_CLASSES
    EVAL_MATRIX(INDICES1{CLASSES_C},1) = CLASSES_C;
    EVAL_MATRIX(INDICES2{CLASSES_C},2) = CLASSES_C;
end

% loop over all test patterns
for ROW = 1:size(EVAL_MATRIX, 1)
    ESTI_CLASS = EVAL_MATRIX(ROW,1);
    TRUE_CLASS = EVAL_MATRIX(ROW,2);

    CONF_MATRIX(TRUE_CLASS+1, ESTI_CLASS+1) = CONF_MATRIX(TRUE_CLASS+1, ESTI_CLASS+1) + 1;
end

for ROW = 1:NUM_CLASSES
    CONF_MATRIX(ROW+1, 2:end) = CONF_MATRIX(ROW+1, 2:end) / sum(CONF_MATRIX(ROW+1, 2:end));
end

%% BUILD LATEX CODE

% the cell with the largest error will be marked red
[VAL,INDICES] = sort(CONF_MATRIX(2:end,2:end),2);
for ROW = 1:NUM_CLASSES
    if INDICES(ROW,end) ~= ROW
        MAX_INDEX(ROW) = INDICES(ROW,end);
    else
        MAX_INDEX(ROW) = INDICES(ROW,end-1);
    end
end

LATEX_TABULAR = ['\n\\begin{table}\n\\begin{footnotesize}\n\\begin{center}\n\\begin{tabular}{' repmat('r',1,NUM_CLASSES+1) '}\n'];
for ROW = 1:NUM_CLASSES
    % add class label before each row
    STR = sprintf('%d ',CONF_MATRIX(ROW+1,1));
    LATEX_TABULAR = strcat(LATEX_TABULAR, STR);
    for COL = 1:NUM_CLASSES
        if COL == ROW
            STR = sprintf('& \\\\textcolor{blue}{%0.1f \\\\%%%%} ', round(CONF_MATRIX(ROW+1,COL+1)*1000)/10);
        else
            if COL == MAX_INDEX(ROW)
                STR = sprintf('& \\\\textcolor{red}{%0.1f \\\\%%%%} ', round(CONF_MATRIX(ROW+1,COL+1)*1000)/10);
            else
                STR = sprintf('& %0.1f \\\\%%%% ', round(CONF_MATRIX(ROW+1,COL+1)*1000)/10);
            end
        end
        LATEX_TABULAR = strcat(LATEX_TABULAR, STR);
    end
    LATEX_TABULAR = strcat(LATEX_TABULAR, '\\\\\n');
end
LATEX_TABULAR = strcat(LATEX_TABULAR, '\\end{tabular}\n\\end{center}\n\\end{footnotesize}\n\\caption{}\n\\label{}\n\\end{table}\n\n');


