function SAMPLES = gmm_samples(PRIORS, MEANS, COVARIANCES, NUMBER_OF_SAMPLES)
%GMM_SAMPLES(PRIORS, MEANS, COVARIANCES, NUMBER_OF_SAMPLES)
% calculates NUMBER_OF_SAMPLES many samples of the gaussian mixture model which is
% given by PRIORS, MEANS and COVARIANCES. MEANS contains the means, where each row is one
% mean centre.
%
% VARIANCES is a 3-dimensional array. COVARIANCES(:, :, 1) contains e.g. the covariance
% matrix for the first pdf part.
%
% Samples is a (NUMBER_OF_SAMPLES x d) huge MATRIX where each row represents one sample.

GMM_DIM     = length(PRIORS);
FEATURE_DIM = size(MEANS, 2);

% check whether dimension of mean vector and dimension of variance matrices is
% appropriate
if (size(MEANS, 1) ~= GMM_DIM || size(COVARIANCES, 3) ~= GMM_DIM...
        || size(COVARIANCES, 1) ~= FEATURE_DIM || size(COVARIANCES, 2) ~= FEATURE_DIM)

    error('Dimensions do not agree! Aborting...');
end

if sum(PRIORS) ~= 1
    error('PRIORS do not sum to one');
end

SAMPLES = [];
for MODE = 1:GMM_DIM
    % calculate number of samples for this mode
    SAMPLE_C = round(PRIORS(MODE) * NUMBER_OF_SAMPLES);
    
    [R,p]=chol(COVARIANCES(:,:,MODE));
    if p>0
        TMP_SAMPLES = randn(SAMPLE_C, FEATURE_DIM) * sqrtm(variance);
    else
        TMP_SAMPLES = randn(SAMPLE_C, FEATURE_DIM) * R;
    end
    
    SAMPLES = [SAMPLES; TMP_SAMPLES + repmat(MEANS(MODE,:),SAMPLE_C,1)];
end